@@ -1,10 +1,13 @@
 #!/usr/bin/env python
 import sys
 
+import gi
+
 from gi.repository import GObject
 GObject.threads_init()
 
 from gi.repository import Gio
+gi.require_version('TelepathyGLib', '0.12')
 from gi.repository import TelepathyGLib
 
 def usage():
