@@ -13,7 +13,7 @@
 #define __EXAMPLE_NO_PROTOCOLS_CONNECTION_MANAGER_H__
 
 #include <glib-object.h>
-#include <telepathy-glib/base-connection-manager.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 G_BEGIN_DECLS
 
