@@ -235,7 +235,7 @@ tp_base_media_call_stream_init (TpBaseMediaCallStream *self)
       TP_TYPE_BASE_MEDIA_CALL_STREAM, TpBaseMediaCallStreamPrivate);
 
   self->priv->local_candidates = g_ptr_array_new_with_free_func (
-      (GDestroyNotify) g_value_array_free);
+      (GDestroyNotify) tp_value_array_free);
   self->priv->username = g_strdup ("");
   self->priv->password = g_strdup ("");
   self->priv->receiving_requests = tp_intset_new ();
@@ -1392,7 +1392,7 @@ tp_base_media_call_stream_set_credentials (TpSvcCallStreamInterfaceMedia *iface,
 
   tp_clear_pointer (&self->priv->local_candidates, g_ptr_array_unref);
   self->priv->local_candidates = g_ptr_array_new_with_free_func (
-      (GDestroyNotify) g_value_array_free);
+      (GDestroyNotify) tp_value_array_free);
 
   g_object_notify (G_OBJECT (self), "local-candidates");
   g_object_notify (G_OBJECT (self), "local-credentials");
@@ -1439,8 +1439,10 @@ tp_base_media_call_stream_add_candidates (TpSvcCallStreamInterfaceMedia *iface,
     {
       GValueArray *c = g_ptr_array_index (accepted_candidates, i);
 
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       g_ptr_array_add (self->priv->local_candidates,
           g_value_array_copy (c));
+      G_GNUC_END_IGNORE_DEPRECATIONS
     }
 
   tp_svc_call_stream_interface_media_emit_local_candidates_added (self,
