@@ -132,6 +132,7 @@ _tp_call_content_new (TpCallChannel *self,
       "object-path", object_path,
       "connection", tp_channel_get_connection ((TpChannel *) self),
       "channel", self,
+      "factory", tp_proxy_get_factory (self),
       NULL);
 }
 
@@ -524,6 +525,8 @@ update_call_members (TpCallChannel *self,
 
   _tp_channel_contacts_queue_prepare_async ((TpChannel *) self,
       contacts, update_call_members_prepared_cb, data);
+
+  g_ptr_array_unref (contacts);
 }
 
 static void
