@@ -40,6 +40,7 @@
 #include "telepathy-glib/deprecated-internal.h"
 #include "telepathy-glib/proxy-internal.h"
 #include "telepathy-glib/simple-client-factory-internal.h"
+#include "telepathy-glib/variant-util-internal.h"
 #include "telepathy-glib/_gen/tp-cli-channel-request-body.h"
 
 /**
@@ -72,20 +73,19 @@
  * indicates that the channel request no longer exists).
  *
  * On failure, the #TpProxy::invalidated signal will be emitted with some
- * other suitable error, usually from the %TP_ERRORS domain.
+ * other suitable error, usually from the %TP_ERROR domain.
  *
  * If the channel dispatcher crashes or exits, the #TpProxy::invalidated
  * signal will be emitted with the domain %TP_DBUS_ERRORS and the error code
  * %TP_DBUS_ERROR_NAME_OWNER_LOST.
  *
- * This proxy is usable but incomplete: accessors for the Account,
- * UserActionTime, PreferredHandler, Requests and Interfaces properties will
- * be added in a later version of telepathy-glib, along with a mechanism
- * similar to tp_connection_call_when_ready().
+ * Creating a #TpChannelRequest directly is deprecated: it
+ * should only be created via a #TpAccountChannelRequest
+ * or a #TpBaseClient.
  *
- * Until suitable convenience methods are implemented, the generic
- * tp_cli_dbus_properties_call_get_all() method can be used to get those
- * properties.
+ * Since 0.16, #TpChannelRequest always has a non-%NULL #TpProxy:factory,
+ * and its #TpProxy:factory will be propagated to the #TpAccount,
+ * #TpConnection and #TpChannel.
  *
  * Since: 0.7.32
  */
@@ -105,10 +105,12 @@ enum {
 enum {
   PROP_CHANNEL_FACTORY = 1,
   PROP_IMMUTABLE_PROPERTIES,
+  PROP_IMMUTABLE_PROPERTIES_VARDICT,
   PROP_ACCOUNT,
   PROP_USER_ACTION_TIME,
   PROP_PREFERRED_HANDLER,
   PROP_HINTS,
+  PROP_HINTS_VARDICT
 };
 
 static guint signals[N_SIGNALS] = { 0 };
@@ -174,6 +176,11 @@ tp_channel_request_get_property (GObject *object,
         g_value_set_boxed (value, self->priv->immutable_properties);
         break;
 
+      case PROP_IMMUTABLE_PROPERTIES_VARDICT:
+        g_value_take_variant (value,
+            tp_channel_request_dup_immutable_properties (self));
+        break;
+
       case PROP_ACCOUNT:
         g_value_set_object (value, tp_channel_request_get_account (self));
         break;
@@ -192,6 +199,10 @@ tp_channel_request_get_property (GObject *object,
         g_value_set_boxed (value, tp_channel_request_get_hints (self));
         break;
 
+      case PROP_HINTS_VARDICT:
+        g_value_take_variant (value, tp_channel_request_dup_hints (self));
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -407,6 +418,28 @@ tp_channel_request_class_init (TpChannelRequestClass *klass)
   g_object_class_install_property (object_class, PROP_IMMUTABLE_PROPERTIES,
       param_spec);
 
+  /**
+   * TpChannelRequest:immutable-properties-vardict:
+   *
+   * The immutable D-Bus properties of this channel request, represented by a
+   * %G_VARIANT_TYPE_VARDICT where the keys are
+   * D-Bus interface name + "." + property name.
+   *
+   * Note that this property is set only if the immutable properties have been
+   * set during the construction of the #TpChannelRequest.
+   *
+   * Read-only except during construction.
+   *
+   * Since: 0.19.10
+   */
+  param_spec = g_param_spec_variant ("immutable-properties-vardict",
+      "Immutable D-Bus properties",
+      "A map D-Bus interface + \".\" + property name => variant",
+      G_VARIANT_TYPE_VARDICT, NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+      PROP_IMMUTABLE_PROPERTIES_VARDICT, param_spec);
+
   /**
    * TpChannelRequest:account:
    *
@@ -474,6 +507,23 @@ tp_channel_request_class_init (TpChannelRequestClass *klass)
       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_HINTS, param_spec);
 
+  /**
+   * TpChannelRequest:hints-vardict:
+   *
+   * A %G_VARIANT_TYPE_VARDICT of metadata provided by
+   * the channel requester; or %NULL if #TpChannelRequest:immutable-properties
+   * is not defined or if no hints have been defined.
+   *
+   * Read-only.
+   *
+   * Since: 0.19.10
+   */
+  param_spec = g_param_spec_variant ("hints-vardict", "Hints", "Hints",
+      G_VARIANT_TYPE_VARDICT, NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_HINTS_VARDICT,
+      param_spec);
+
   /**
    * TpChannelRequest::succeeded:
    * @self: the channel request proxy
@@ -544,7 +594,7 @@ tp_channel_request_init_known_interfaces (void)
       tp_proxy_or_subclass_hook_on_interface_add (tp_type,
           tp_cli_channel_request_add_signals);
       tp_proxy_subclass_add_error_mapping (tp_type,
-          TP_ERROR_PREFIX, TP_ERRORS, TP_TYPE_ERROR);
+          TP_ERROR_PREFIX, TP_ERROR, TP_TYPE_ERROR);
 
       g_once_init_leave (&once, 1);
     }
@@ -567,6 +617,8 @@ tp_channel_request_init_known_interfaces (void)
  * Returns: a new reference to an channel request proxy, or %NULL if
  *    @object_path is not syntactically valid or the channel dispatcher is
  *    not running
+ * Deprecated: Since 0.19.9. New code should get #TpChannelRequest objects
+ *  from a #TpBaseClient
  */
 TpChannelRequest *
 tp_channel_request_new (TpDBusDaemon *bus_daemon,
@@ -661,6 +713,28 @@ tp_channel_request_get_immutable_properties (TpChannelRequest *self)
   return self->priv->immutable_properties;
 }
 
+/**
+ * tp_channel_request_dup_immutable_properties:
+ * @self: a #TpChannelRequest
+ *
+ * Return the #TpChannelRequest:immutable-properties-vardict property.
+ *
+ * Returns: (transfer full): the value of
+ * #TpChannelRequest:immutable-properties-vardict
+ *
+ * Since: 0.19.10
+ */
+GVariant *
+tp_channel_request_dup_immutable_properties (TpChannelRequest *self)
+{
+  g_return_val_if_fail (TP_IS_CHANNEL_REQUEST (self), NULL);
+
+  if (self->priv->immutable_properties == NULL)
+    return NULL;
+
+  return _tp_asv_to_vardict (self->priv->immutable_properties);
+}
+
 void
 _tp_channel_request_ensure_immutable_properties (TpChannelRequest *self,
     GHashTable *immutable_properties)
@@ -773,3 +847,28 @@ tp_channel_request_get_hints (TpChannelRequest *self)
   return tp_asv_get_boxed (self->priv->immutable_properties,
       TP_PROP_CHANNEL_REQUEST_HINTS, TP_HASH_TYPE_STRING_VARIANT_MAP);
 }
+
+/**
+ * tp_channel_request_dup_hints:
+ * @self: a #TpChannelRequest
+ *
+ * Return the #TpChannelRequest:hints-vardict property
+ *
+ * Returns: (transfer full): the value of #TpChannelRequest:hints-vardict
+ *
+ * Since: 0.19.10
+ */
+GVariant *
+tp_channel_request_dup_hints (TpChannelRequest *self)
+{
+  const GHashTable *hints;
+
+  g_return_val_if_fail (TP_IS_CHANNEL_REQUEST (self), NULL);
+
+  hints = tp_channel_request_get_hints (self);
+
+  if (hints == NULL)
+    return NULL;
+
+  return _tp_asv_to_vardict (hints);
+}
