@@ -65,7 +65,8 @@
 /**
  * tp_g_socket_address_from_variant:
  * @type: a Telepathy socket address type
- * @variant: an initialised #GValue containing an address variant
+ * @variant: an initialised #GValue containing an address variant,
+ *  as encoded by dbus-glib
  * @error: return location for a #GError (or NULL)
  *
  * Converts an address variant stored in a #GValue into a #GSocketAddress that
@@ -87,7 +88,7 @@ tp_g_socket_address_from_variant (TpSocketAddressType type,
       case TP_SOCKET_ADDRESS_TYPE_UNIX:
         if (!G_VALUE_HOLDS (variant, DBUS_TYPE_G_UCHAR_ARRAY))
           {
-            g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+            g_set_error (error, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
                 "variant is %s not DBUS_TYPE_G_UCHAR_ARRAY",
                 G_VALUE_TYPE_NAME (variant));
 
@@ -108,7 +109,7 @@ tp_g_socket_address_from_variant (TpSocketAddressType type,
       case TP_SOCKET_ADDRESS_TYPE_ABSTRACT_UNIX:
         if (!G_VALUE_HOLDS (variant, DBUS_TYPE_G_UCHAR_ARRAY))
           {
-            g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+            g_set_error (error, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
                 "variant is %s not DBUS_TYPE_G_UCHAR_ARRAY",
                 G_VALUE_TYPE_NAME (variant));
 
@@ -129,7 +130,7 @@ tp_g_socket_address_from_variant (TpSocketAddressType type,
         if (type == TP_SOCKET_ADDRESS_TYPE_IPV4 &&
             !G_VALUE_HOLDS (variant, TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4))
           {
-            g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+            g_set_error (error, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
                 "variant is %s not TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4",
                 G_VALUE_TYPE_NAME (variant));
 
@@ -138,7 +139,7 @@ tp_g_socket_address_from_variant (TpSocketAddressType type,
         else if (type == TP_SOCKET_ADDRESS_TYPE_IPV6 &&
             !G_VALUE_HOLDS (variant, TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV6))
           {
-            g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+            g_set_error (error, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
                 "variant is %s not TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV6",
                 G_VALUE_TYPE_NAME (variant));
 
@@ -147,8 +148,10 @@ tp_g_socket_address_from_variant (TpSocketAddressType type,
         else
           {
             GValueArray *array = g_value_get_boxed (variant);
+            G_GNUC_BEGIN_IGNORE_DEPRECATIONS
             GValue *hostv = g_value_array_get_nth (array, 0);
             GValue *portv = g_value_array_get_nth (array, 1);
+            G_GNUC_END_IGNORE_DEPRECATIONS
             GInetAddress *address;
             const char *host;
             guint16 port;
@@ -167,7 +170,7 @@ tp_g_socket_address_from_variant (TpSocketAddressType type,
         break;
 
       default:
-        g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+        g_set_error (error, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
             "Unknown TpSocketAddressType (%i)",
             type);
 
@@ -184,7 +187,7 @@ tp_g_socket_address_from_variant (TpSocketAddressType type,
  * @error: return location for a #GError (or NULL)
  *
  * Converts a #GSocketAddress to a #GValue address variant that can be used
- * with Telepathy.
+ * with Telepathy and dbus-glib.
  *
  * Returns: a newly allocated #GValue, free with tp_g_value_slice_free()
  */
@@ -268,7 +271,7 @@ tp_address_variant_from_g_socket_address (GSocketAddress *address,
         break;
 
       default:
-        g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+        g_set_error (error, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
             "Unknown GSocketAddressFamily %i",
             g_socket_address_get_family (address));
 
@@ -281,6 +284,69 @@ tp_address_variant_from_g_socket_address (GSocketAddress *address,
   return variant;
 }
 
+/**
+ * tp_g_socket_address_from_g_variant:
+ * @type: a Telepathy socket address type
+ * @variant: a socket address as encoded by Telepathy according to @type
+ * @error: return location for a #GError (or %NULL)
+ *
+ * Converts an address variant stored in a #GVariant into a #GSocketAddress
+ * that can be used to make a socket connection with GIO.
+ *
+ * If @variant is a floating reference, this function takes ownership
+ * of it.
+ *
+ * Returns: a newly allocated #GSocketAddress for the given variant, or %NULL
+ * on error
+ *
+ * Since: 0.19.10
+ */
+GSocketAddress *
+tp_g_socket_address_from_g_variant (TpSocketAddressType type,
+    GVariant *variant,
+    GError **error)
+{
+  GValue value = G_VALUE_INIT;
+  GSocketAddress *ret;
+
+  g_variant_ref_sink (variant);
+  dbus_g_value_parse_g_variant (variant, &value);
+  g_variant_unref (variant);
+  ret = tp_g_socket_address_from_variant (type, &value, error);
+  g_value_unset (&value);
+  return ret;
+}
+
+/**
+ * tp_address_g_variant_from_g_socket_address:
+ * @address: a #GSocketAddress to convert
+ * @type: optional return of the Telepathy socket type (or NULL)
+ * @error: return location for a #GError (or NULL)
+ *
+ * Converts a #GSocketAddress to a #GVariant address variant that can be used
+ * with Telepathy.
+ *
+ * Returns: (transfer none): a new variant with a floating reference, or %NULL
+ *
+ * Since: 0.19.10
+ */
+GVariant *
+tp_address_g_variant_from_g_socket_address (GSocketAddress *address,
+    TpSocketAddressType *type,
+    GError **error)
+{
+  GValue *value = tp_address_variant_from_g_socket_address (address,
+      type, error);
+  GVariant *ret;
+
+  if (value == NULL)
+    return NULL;
+
+  ret = dbus_g_value_build_g_variant (value);
+  tp_g_value_slice_free (value);
+  return ret;
+}
+
 #ifdef HAVE_GIO_UNIX
 static gboolean
 _tp_unix_connection_send_credentials_with_byte (GUnixConnection *connection,
@@ -337,7 +403,7 @@ _tp_unix_connection_send_credentials_with_byte (GUnixConnection *connection,
 #endif
 
 /**
- * tp_unix_connection_send_credentials_with_byte
+ * tp_unix_connection_send_credentials_with_byte:
  * @connection: a #GUnixConnection
  * @byte: the byte to send with the credentials
  * @cancellable: (allow-none): a #GCancellable, or %NULL
