@@ -371,7 +371,7 @@ test_prepare_success (Test *test,
   g_main_loop_run (test->mainloop);
 
   /* the obvious accessors */
-  g_assert (tp_account_is_prepared (test->account, TP_ACCOUNT_FEATURE_CORE));
+  g_assert (tp_proxy_is_prepared (test->account, TP_ACCOUNT_FEATURE_CORE));
   g_assert (tp_account_is_enabled (test->account));
   assert_boolprop (test->account, "enabled", TRUE);
   g_assert (tp_account_is_valid (test->account));
@@ -689,6 +689,13 @@ test_addressing (Test *test,
         "xmpp"));
 }
 
+static void
+avatar_changed_cb (TpAccount *account,
+    Test *test)
+{
+  g_main_loop_quit (test->mainloop);
+}
+
 static void
 test_avatar (Test *test,
     gconstpointer mode)
@@ -714,6 +721,27 @@ test_avatar (Test *test,
   g_assert_cmpstr (((char *) blob->data), ==, ":-)");
 
   tp_clear_object (&test->result);
+
+  /* change the avatar */
+  g_signal_connect (test->account, "avatar-changed",
+      G_CALLBACK (avatar_changed_cb), test);
+
+  tp_tests_simple_account_set_avatar (test->account_service, ":-(");
+  g_main_loop_run (test->mainloop);
+
+  tp_account_get_avatar_async (test->account,
+      tp_tests_result_ready_cb, &test->result);
+  tp_tests_run_until_result (&test->result);
+
+  blob = tp_account_get_avatar_finish (
+      test->account, test->result, &error);
+  g_assert_no_error (error);
+
+  g_assert (blob != NULL);
+  g_assert_cmpuint (blob->len, ==, 4);
+  g_assert_cmpstr (((char *) blob->data), ==, ":-(");
+
+  tp_clear_object (&test->result);
 }
 
 static void
@@ -738,7 +766,7 @@ test_connection (Test *test,
       account_prepare_cb, test);
   g_main_loop_run (test->mainloop);
 
-  g_assert (tp_account_is_prepared (test->account, TP_ACCOUNT_FEATURE_CORE));
+  g_assert (tp_proxy_is_prepared (test->account, TP_ACCOUNT_FEATURE_CORE));
 
   /* a connection turns up */
 
@@ -902,7 +930,6 @@ int
 main (int argc,
       char **argv)
 {
-  g_type_init ();
   tp_tests_abort_after (10);
   tp_debug_set_flags ("all");
 
@@ -953,7 +980,7 @@ main (int argc,
 
   g_test_add ("/account/reconnect", Test, NULL, setup_service, test_reconnect,
       teardown_service);
-  g_test_add ("/account/reconnect", Test, "vardict", setup_service,
+  g_test_add ("/account/reconnect/vardict", Test, "vardict", setup_service,
       test_reconnect, teardown_service);
 
   g_test_add ("/account/prepare/success", Test, NULL, setup_service,
@@ -964,16 +991,16 @@ main (int argc,
 
   g_test_add ("/account/storage", Test, "first", setup_service, test_storage,
       teardown_service);
-  g_test_add ("/account/storage", Test, "later", setup_service, test_storage,
-      teardown_service);
+  g_test_add ("/account/storage/later", Test, "later", setup_service,
+      test_storage, teardown_service);
 
   g_test_add ("/account/avatar", Test, NULL, setup_service, test_avatar,
       teardown_service);
 
   g_test_add ("/account/addressing", Test, "first", setup_service,
       test_addressing, teardown_service);
-  g_test_add ("/account/addressing", Test, "later", setup_service,
+  g_test_add ("/account/addressing/later", Test, "later", setup_service,
       test_addressing, teardown_service);
 
-  return g_test_run ();
+  return tp_tests_run_with_bus ();
 }
