@@ -75,7 +75,6 @@ setup (Test *test,
 {
   DBusConnection *libdbus;
 
-  g_type_init ();
   tp_debug_set_flags ("all");
 
   test->mainloop = g_main_loop_new (NULL, FALSE);
@@ -440,5 +439,5 @@ main (int argc,
       test_immutable_properties, teardown);
   g_test_add ("/cr/properties", Test, NULL, setup, test_properties, teardown);
 
-  return g_test_run ();
+  return tp_tests_run_with_bus ();
 }
