@@ -202,7 +202,6 @@ setup (Test *test,
   GQuark features[] = { TP_CONNECTION_FEATURE_CONNECTED, 0 };
   GType conn_type = GPOINTER_TO_SIZE (data);
 
-  g_type_init ();
   tp_debug_set_flags ("all");
   test->dbus = tp_tests_dbus_daemon_dup_or_die ();
 
@@ -383,8 +382,6 @@ int
 main (int argc,
       char **argv)
 {
-  g_type_init ();
-
   tp_tests_abort_after (5);
   g_test_init (&argc, &argv, NULL);
 
@@ -398,5 +395,5 @@ main (int argc,
       GSIZE_TO_POINTER (TP_TESTS_TYPE_SIMPLE_CONNECTION),
       setup, test_balance_unknown, teardown);
 
-  return g_test_run ();
+  return tp_tests_run_with_bus ();
 }
