@@ -585,7 +585,6 @@ main (int argc,
       char **argv)
 {
   tp_tests_abort_after (10);
-  g_type_init ();
   tp_debug_set_flags ("all");
 
   g_test_init (&argc, &argv, NULL);
@@ -614,5 +613,5 @@ main (int argc,
   g_test_add ("/file-transfer-channel/provide/cancel", Test, NULL, setup,
       test_cancel_transfer, teardown);
 
-  return g_test_run ();
+  return tp_tests_run_with_bus ();
 }
