@@ -13,9 +13,7 @@
 #define __TP_TESTS_CONTACTS_CONN_H__
 
 #include <glib-object.h>
-#include <telepathy-glib/base-connection.h>
-#include <telepathy-glib/contacts-mixin.h>
-#include <telepathy-glib/presence-mixin.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 #include "simple-conn.h"
 #include "contact-list-manager.h"
@@ -74,7 +72,7 @@ typedef enum {
   (G_TYPE_INSTANCE_GET_CLASS ((obj), TP_TESTS_TYPE_CONTACTS_CONNECTION, \
                               TpTestsContactsConnectionClass))
 
-TestContactListManager *tp_tests_contacts_connection_get_contact_list_manager (
+TpTestsContactListManager *tp_tests_contacts_connection_get_contact_list_manager (
     TpTestsContactsConnection *self);
 
 void tp_tests_contacts_connection_change_aliases (
