@@ -16,8 +16,11 @@
 #include "echo-chan.h"
 
 #include <telepathy-glib/telepathy-glib.h>
-#include <telepathy-glib/channel-iface.h>
-#include <telepathy-glib/svc-channel.h>
+#include <telepathy-glib/telepathy-glib-dbus.h>
+
+/* This is for text-mixin unit tests, others should be using ExampleEcho2Channel
+ * which uses newer TpMessageMixin */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
 static void text_iface_init (gpointer iface, gpointer data);
 static void destroyable_iface_init (gpointer iface, gpointer data);
@@ -32,9 +35,16 @@ G_DEFINE_TYPE_WITH_CODE (TpTestsEchoChannel,
 
 /* type definition stuff */
 
-static const char * tp_tests_echo_channel_interfaces[] = {
-    TP_IFACE_CHANNEL_INTERFACE_DESTROYABLE,
-    NULL
+static GPtrArray *
+tp_tests_echo_channel_get_interfaces (TpBaseChannel *self)
+{
+  GPtrArray *interfaces;
+
+  interfaces = TP_BASE_CHANNEL_CLASS (tp_tests_echo_channel_parent_class)->
+    get_interfaces (self);
+
+  g_ptr_array_add (interfaces, TP_IFACE_CHANNEL_INTERFACE_DESTROYABLE);
+  return interfaces;
 };
 
 static void
@@ -125,7 +135,7 @@ tp_tests_echo_channel_class_init (TpTestsEchoChannelClass *klass)
 
   base_class->channel_type = TP_IFACE_CHANNEL_TYPE_TEXT;
   base_class->target_handle_type = TP_HANDLE_TYPE_CONTACT;
-  base_class->interfaces = tp_tests_echo_channel_interfaces;
+  base_class->get_interfaces = tp_tests_echo_channel_get_interfaces;
   base_class->close = channel_close;
 
   tp_text_mixin_class_init (object_class,
@@ -211,3 +221,5 @@ destroyable_iface_init (gpointer iface,
   IMPLEMENT (destroy);
 #undef IMPLEMENT
 }
+
+G_GNUC_END_IGNORE_DEPRECATIONS
