@@ -13,8 +13,7 @@
 #define __TP_FILE_TRANSFER_CHAN_H__
 
 #include <glib-object.h>
-#include <telepathy-glib/base-channel.h>
-#include <telepathy-glib/base-connection.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 G_BEGIN_DECLS
 
