@@ -12,7 +12,7 @@
 
 #include "my-conn-proxy.h"
 
-#include <telepathy-glib/proxy-internal.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 G_DEFINE_TYPE  (TpTestsMyConnProxy, tp_tests_my_conn_proxy,
     TP_TYPE_CONNECTION)
