@@ -12,7 +12,7 @@
 #define __TP_TESTS_SIMPLE_CONNECTION_MANAGER_H__
 
 #include <glib-object.h>
-#include <telepathy-glib/base-connection-manager.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 G_BEGIN_DECLS
 
